@@ -3,9 +3,6 @@
 #![allow(unused_assignments)]
 #![allow(unused_must_use)]
 
-#[macro_use]
-extern crate lazy_static;
-
 pub mod appender;
 pub mod bencher;
 pub mod consts;
